rem -------------------------------------------------------------------------
rem        Trivadis AG, Baden/Basel/Bern/Lausanne/Zrich
rem                     Frankfurt/Freiburg i.Br./Mnchen/Stuttgart
rem                     Switzerland/Germany Internet: http://www.trivadis.com
rem -------------------------------------------------------------------------
rem   $Header$
rem   **********************************************************************
rem
rem   Group/Privileges.: DBA
rem   Script-Name......: sdtabfbi.sql
rem   Developer........: Martin Wunderli martin.wunderli@trivadis.com
rem   Date.............: 11.10.2000
rem   Version..........: Oracle10g
rem   Usage............: Shows the indexes for a given table (or many) 
rem                      incl. the function based indexes (FBI)
rem                      For FBI-Columns not SYS_NCnnn is shown, but
rem                      the LONG DEFAULT$ coming from SYS.COL$ through DBA_IND_EXPRESSIONS 
rem   Input parameters.: Table_Name or Wildcard / Owner or Wildcard
rem   Output.......... : 
rem   Called by........:
rem   Remarks..........: Oracle stores the columns of an FBI in a LONG (thanks :-). So it is not
rem                      so easy to display the columns and we create in this script first a
rem                      function as to fetch the LONG using DBMS_SQL. Once this function created
rem                      users could (should) comment the CREATE OR REPLACE FUNCTION as not to
rem                      recreate the function each time (SYS.SOURCE$ will thank you)
rem
rem ----------------------------------------------------------------------------
rem HISTORY:
rem ============================================================================
rem
rem $Log$
rem Revision 1.1  2003/09/09 11:38:12  ank
rem - OK for 10.1
rem
rem Revision 1.5  2002/09/09 16:58:52  far
rem 9.2 fhig checked by ChA
rem
rem Revision 1.3  2001/09/28 10:31:55  ank
rem - Fixed the ORDER BY (YaN)
rem
rem Revision 1.2  2001/07/22 12:59:28  ank
rem - Load initial release Oracle9i 9.0.x
rem
rem
rem
rem Changes:
rem DD.MM.YYYY Consultant Change
rem -----------------------------------------------------------------------
rem 11.10.2000 MaW        Created for use by a dba (99% based on sotabfbi by AnK)
rem 23.01.2001 ChA        Added some hints for better performances
rem 25.07.2002 ChA        Bug correction: the owner was not part of the FKs
rem -----------------------------------------------------------------------
rem
rem -----------------------------------------------------------------------
rem
rem
rem
set echo off
set verify off
@@fopauon
column index_name       format a20 word_wrap    heading 'Index_Name (Part)'
column owner            format a12              heading 'Index Owner'
column type             format a3               heading 'Typ'
column uni              format a3               heading 'Uni'
column tablespace_name  format a10 wrap         heading 'Table Space'
column table_name       noprint                 new_value tab
column column_name      format a18 wrap         heading 'Column Name'
column distinct_keys    format 9999999          heading 'DistKeys'
column column_position  noprint
ttitle center 'Indexes for table 'tab skip 2
break on table_name skip page -
      on index_name skip 2 -
      on type -
      on uni -
      on distinct_keys  -
      on tablespace_name
create or replace function fkt_get_idx_expression (v_iname VARCHAR2, v_tname VARCHAR2, v_pos NUMBER)
 RETURN VARCHAR2
AS
begin
declare
v_l varchar2(4000);
 /* for dbms_sql: */
vDummy          INTEGER;
vCursor         INTEGER;
vLongChunk      VARCHAR2(32767);  -- Return chunk into this variable
vChunkSize      INTEGER := 32767; -- How long should the chunk be
vChunkReturned  INTEGER;       -- Length of last chunk fetched
vPosition       INTEGER := 0;  -- Current position to fetch from
vSQL            VARCHAR2(2000);

begin
   vSQL        := 'SELECT column_expression '      ||
                 '  FROM dba_ind_expressions '||
                 '  WHERE index_name = :iname ' ||
                 '  AND table_name = :tname ' ||
                 '  AND column_position = :cpos ';
   vPosition := 0;
   vCursor     := DBMS_SQL.Open_Cursor;
   DBMS_SQL.Parse(vCursor,vSQL,DBMS_SQL.native);
   DBMS_SQL.Bind_Variable(vCursor,'iname',v_iname);
   DBMS_SQL.Bind_Variable(vCursor,'tname',v_tname);
   DBMS_SQL.Bind_Variable(vCursor,'cpos',v_pos);

   -- Define the LONGs in the SELECT Liste by position
   DBMS_SQL.Define_Column_Long(vCursor,1);
   vDummy := DBMS_SQL.Execute(vCursor);
   vDummy := DBMS_SQL.Fetch_Rows(vCursor);
   v_l := '';
   loop  /* loop through the LONG */
    dbms_sql.column_value_long(
       vCursor,
       1,
       vChunkSize,
       vPosition,
       vLongChunk,
       vChunkReturned);
    v_l := v_l||vLongChunk;

    vPosition := vPosition + vChunkSize;
    -- If we couldn't fill the last chunk
    -- then we are finished...
    EXIT WHEN vChunkReturned < vChunkSize;
   end loop; /* end loop through the LONG */
   dbms_sql.close_cursor(vCursor);
   return v_l;
end;
end;
/
show errors
PROMPT
PROMPT Please enter table name, wildcards allowed, eg.: EMP, E% or %
PROMPT
accept table_name char prompt "Table Name <%>: " default "%"
PROMPT
PROMPT Please enter table owner, wildcards allowed, eg.: SCOTT, S% or %
PROMPT
accept owner char prompt "Table Owner <%>: " default "%"
select 	/*+ ordered 
            no_merge(i) 
            no_merge(c) use_hash(c) */ 
        /* first non-partitioned indexes */
                i.index_name,i.owner,i.table_name,c.column_name||' '||decode(descend,'ASC','',descend) column_name,
		decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
		                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
		i.tablespace_name, i.distinct_keys, c.column_position 
   	from  dba_indexes i, dba_ind_columns c
        where i.table_name = c.table_name
        and i.index_name = c.index_name
        and i.owner = c.index_owner
    	and i.table_name like upper('&&Table_Name')
        and i.table_owner like upper('&&Owner')
        and i.partitioned = 'NO'
        and c.column_name not like 'SYS_NC%' /* FBIs handled later */
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) */ 
        /* and now partitioned indexes */ 
        i.index_name||' ('||ip.partition_name||') '||
                    ' (P:'||p.locality||' '||p.alignment||')' index_name,i.owner,
        c.table_name,c.column_name||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  dba_indexes i, dba_ind_partitions ip,
              dba_ind_columns c, dba_part_indexes p
        where i.table_name = c.table_name
        and i.table_owner = c.table_owner
        and p.index_name = ip.index_name
        and p.owner = ip.index_owner
        and i.index_name = ip.index_name
        and i.owner = ip.index_owner
        and ip.index_name = c.index_name
        and ip.index_owner = c.index_owner
        and i.table_name like upper('&&Table_Name')
        and i.table_owner like upper('&&Owner')
        and ip.tablespace_name is not null /* Comp. part. Indexes */
        and c.column_name not like 'SYS_NC%' /* FBIs handled later */
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) */ 
        /* and now sub-partitioned indexes */ 
        i.index_name||' ('||ip.subpartition_name||') '||
                    ' (SP:'||p.locality||' '||p.alignment||')' index_name,i.owner,
        c.table_name,c.column_name||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  dba_indexes i, dba_ind_subpartitions ip,
              dba_ind_columns c, dba_part_indexes p
        where i.table_name = c.table_name
        and i.table_owner = c.table_owner
        and p.index_name = ip.index_name
        and p.owner = ip.index_owner
        and i.index_name = ip.index_name
        and i.owner = ip.index_owner
        and ip.index_name = c.index_name
        and ip.index_owner = c.index_owner
        and i.table_name like upper('&&Table_Name')
        and i.table_owner like upper('&&Owner')
        and c.column_name not like 'SYS_NC%' /* FBIs handled later */
 union
select /*+ ordered 
            no_merge(i) 
            no_merge(l) use_hash(l) */ 
       /* and now the LOB-Indexes */
       l.index_name,i.owner, l.table_name, l.column_name,'LOB' TYPE,
       'LOB' UNI, i.tablespace_name,
       -1,1
       from dba_indexes i, dba_lobs l
       where l.index_name = i.index_name
       and   l.table_name = i.table_name
       and   l.owner = i.table_owner
       and   l.table_name like upper('&&Table_Name')
        and i.table_owner like upper('&&Owner')
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(c) use_hash(c)
            no_merge(e) use_hash(e) */ 
        /* and now the FBIs on normal tables */
                i.index_name,i.owner,i.table_name,fkt_get_idx_expression(i.index_name,i.table_name,c.column_position)||' '||decode(descend,'ASC','',descend) column_name,
		decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
		                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
		i.tablespace_name, i.distinct_keys, c.column_position
   	from  dba_indexes i, dba_ind_columns c, dba_ind_expressions e
  	where i.table_name = c.table_name
        and   i.owner = c.index_owner
        and   e.table_name = i.table_name
        and   e.table_owner = i.table_owner
        and   e.index_name = i.index_name
        and   e.index_owner = i.owner
    	and   i.index_name = c.index_name
    	and   i.owner = c.index_owner
        and   e.column_position = c.column_position
    	and i.table_name like upper('&&Table_Name')
        and i.table_owner like upper('&&Owner')
        and i.partitioned = 'NO'
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) 
            no_merge(e) use_hash(e) */ 
        /* and now partitioned FBIs */ 
        i.index_name||' ('||ip.partition_name||') '||
                    ' (P:'||p.locality||' '||p.alignment||')' index_name,i.owner,
        c.table_name,fkt_get_idx_expression(i.index_name,i.table_name,c.column_position)||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  dba_indexes i, dba_ind_partitions ip,
              dba_ind_columns c, dba_part_indexes p, dba_ind_expressions e
        where i.table_name = c.table_name
        and   i.table_owner = c.table_owner
        and   e.table_name = i.table_name
        and   e.table_owner = i.table_owner
        and   e.index_name = i.index_name
        and   e.index_owner = i.owner
        and   e.column_position = c.column_position
        and p.index_name = ip.index_name
        and p.owner = ip.index_owner
        and i.index_name = ip.index_name
        and i.owner = ip.index_owner
        and ip.index_name = c.index_name
        and ip.index_owner = c.index_owner
        and i.table_name like upper('&&Table_Name')
        and i.table_owner like upper('&&Owner')
        and ip.tablespace_name is not null /* Comp. part. Indexes */
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) 
            no_merge(e) use_hash(e) */ 
        /* and now sub-partitioned FBIs */ 
        i.index_name||' ('||ip.subpartition_name||') '||
                    ' (SP:'||p.locality||' '||p.alignment||')' index_name,i.owner,
        c.table_name,fkt_get_idx_expression(i.index_name,i.table_name,c.column_position)||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  dba_indexes i, dba_ind_subpartitions ip,
              dba_ind_columns c, dba_part_indexes p, dba_ind_expressions e
        where i.table_name = c.table_name
        and   i.table_owner = c.table_owner
        and   e.table_name = i.table_name
        and   e.table_owner = i.table_owner
        and   e.index_name = i.index_name
        and   e.index_owner = i.owner
        and   e.column_position = c.column_position
        and p.index_name = ip.index_name
        and p.owner = ip.index_owner
        and i.index_name = ip.index_name
        and i.owner = ip.index_owner
        and ip.index_name = c.index_name
        and ip.index_owner = c.index_owner
        and i.table_name like upper('&&Table_Name')
        and i.table_owner like upper('&&Owner')
order by 2, 5 desc, 1, 9;

set pause off
ttitle off
clear break compute     
column index_name	CLEAR
column type		CLEAR
column uni              CLEAR
column tablespace_name	CLEAR
column table_name	CLEAR
column column_name	CLEAR
column distinct_keys	CLEAR
column column_position  CLEAR
undefine table_name
undefine owner
