rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem $Id: sotabfbi.sql 35 2008-11-20 18:56:30Z cha $
rem ---------------------------------------------------------------------------
rem  Group/Privileges.: User
rem  Script Name......: sotabfbi.sql
rem  Developer........: Andri Kisseleff (AnK)
rem  Date.............: 03.02.2000
rem  Version..........: Oracle Database 11g
rem  Description......: Shows the indexes for given table(s) incl. the 
rem                     function-based indexes (FBI). For FBI columns, instead
rem                     of showing SYS_NCnnn, the LONG DEFAULT$ coming from
rem                     SYS.COL$ through USER_IND_EXPRESSIONS is shown.
rem  Usage............: 
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: Oracle stores the columns of an FBI in a LONG (thanks :-).
rem                     So it is not so easy to display the columns and we create
rem                     in this script first a function as to fetch the LONG
rem                     using DBMS_SQL. Once this function created users could
rem                     (should) comment the CREATE OR REPLACE FUNCTION as not
rem                     to recreate the function each time (SYS.SOURCE$ will
rem                     thank you).
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem 23.01.2001 ChA       Added some hints for better performances
rem 27.08.2002 ChA       OK for Oracle9i R2
rem 30.08.2002 FaR       getestet/angepasst fr 9.2
rem 09.09.2003 AnK       OK for 10.1 (Bitmap idx. on IOTs, List part. IOT)
rem 20.11.2008 ChA       Fixed header + OK for 11g + Handle virtual columns
rem ---------------------------------------------------------------------------

store set temp.tmp replace
set echo off
set verify off
set linesize 100
@@fopauon
column index_name	format a40 word_wrap	heading 'Index_Name (Partition)' 
column type		format a3		heading 'Typ'
column uni		format a3		heading 'Uni'
column tablespace_name	format a10 wrap		heading 'Tablespace'
column table_name	noprint			new_value tab
column column_name	format a25 wrap		heading 'Column Name'
column distinct_keys	format 9G999G999G999	heading 'Distinct Keys'
column column_position  noprint
ttitle center 'Indexes for table 'tab skip 2
break on table_name skip page -
      on index_name skip 2 -
      on type -
      on uni -
      on distinct_keys  -
      on tablespace_name
create or replace function fkt_get_idx_expression (v_iname VARCHAR2, v_tname VARCHAR2, v_pos NUMBER)
 RETURN VARCHAR2
AS
begin
declare
v_l varchar2(4000);
 /* for dbms_sql: */
vDummy          INTEGER;
vCursor         INTEGER;
vLongChunk      VARCHAR2(32767);  -- Return chunk into this variable
vChunkSize      INTEGER := 32767; -- How long should the chunk be
vChunkReturned  INTEGER;       -- Length of last chunk fetched
vPosition       INTEGER := 0;  -- Current position to fetch from
vSQL            VARCHAR2(2000);

begin
   vSQL        := 'SELECT column_expression '      ||
                 '  FROM user_ind_expressions '||
                 '  WHERE index_name = :iname ' ||
                 '  AND table_name = :tname ' ||
                 '  AND column_position = :cpos ';
   vPosition := 0;
   vCursor     := DBMS_SQL.Open_Cursor;
   DBMS_SQL.Parse(vCursor,vSQL,DBMS_SQL.native);
   DBMS_SQL.Bind_Variable(vCursor,'iname',v_iname);
   DBMS_SQL.Bind_Variable(vCursor,'tname',v_tname);
   DBMS_SQL.Bind_Variable(vCursor,'cpos',v_pos);

   -- Define the LONGs in the SELECT Liste by position
   DBMS_SQL.Define_Column_Long(vCursor,1);
   vDummy := DBMS_SQL.Execute(vCursor);
   vDummy := DBMS_SQL.Fetch_Rows(vCursor);
   v_l := '';
   loop  /* loop through the LONG */
    dbms_sql.column_value_long(
       vCursor,
       1,
       vChunkSize,
       vPosition,
       vLongChunk,
       vChunkReturned);
    v_l := v_l||vLongChunk;
    vPosition := vPosition + vChunkSize;
    -- If we couldn't fill the last chunk
    -- then we are finished...
    EXIT WHEN vChunkReturned < vChunkSize;
   end loop; /* end loop through the LONG */
   dbms_sql.close_cursor(vCursor);
   return v_l;
end;
end;
/
show errors
PROMPT 
PROMPT Please enter table name, wildcards allowed
PROMPT
PROMPT eg.: EMP, E% or %
PROMPT
accept table_name char prompt "Table Name <%>: " default "%"
select 	/*+ ordered 
            no_merge(i) 
            no_merge(c) use_hash(c) */ 
        /* first non-partitioned indexes */
                i.index_name,i.table_name,c.column_name||' '||decode(descend,'ASC','',descend) column_name,
		decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
		                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
		i.tablespace_name, i.distinct_keys, c.column_position 
   	from  user_indexes i, user_ind_columns c
  	where i.table_name = c.table_name
    	and i.index_name = c.index_name
    	and i.table_name like upper('&&Table_Name')
        and i.partitioned = 'NO'
        and c.column_name not like 'SYS_NC%' /* FBIs handled later */
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) */ 
        /* and now partitioned indexes */ 
        i.index_name||' ('||ip.partition_name||') '||
                    ' (P:'||p.locality||' '||p.alignment||')' index_name,
        c.table_name,c.column_name||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  user_indexes i, user_ind_partitions ip,
              user_ind_columns c, user_part_indexes p
        where i.table_name = c.table_name
        and p.index_name = ip.index_name
        and i.index_name = ip.index_name
        and ip.index_name = c.index_name
        and i.table_name like upper('&&Table_Name')
        and ip.tablespace_name is not null /* Comp. part. Indexes */
        and c.column_name not like 'SYS_NC%' /* FBIs handled later */
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) */ 
        /* and now sub-partitioned indexes */ 
        i.index_name||' ('||ip.subpartition_name||') '||
                    ' (SP:'||p.locality||' '||p.alignment||')' index_name,
        c.table_name,c.column_name||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  user_indexes i, user_ind_subpartitions ip,
              user_ind_columns c, user_part_indexes p
        where i.table_name = c.table_name
        and p.index_name = ip.index_name
        and i.index_name = ip.index_name
        and ip.index_name = c.index_name
        and i.table_name like upper('&&Table_Name')
        and c.column_name not like 'SYS_NC%' /* FBIs handled later */
 union
select /*+ ordered 
            no_merge(i) 
            no_merge(l) use_hash(l) */ 
       /* and now the LOB-Indexes */
       l.index_name, l.table_name, l.column_name,'LOB' TYPE,
       'LOB' UNI, i.tablespace_name,
       -1,1
       from user_indexes i, user_lobs l
       where l.index_name = i.index_name
       and   l.table_name = i.table_name
       and   l.table_name like upper('&&Table_Name')
 union
select /*+ ordered 
            no_merge(i) 
            no_merge(c) use_hash(c)
            no_merge(e) use_hash(e) */ 
       /* and now the FBIs on normal tables */
                i.index_name,
                i.table_name,
                decode(tc.hidden_column,'NO',c.column_name,fkt_get_idx_expression(i.index_name,i.table_name,c.column_position))||' '||decode(descend,'ASC','',descend) column_name,
		decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
		                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
		i.tablespace_name, i.distinct_keys, c.column_position
   	from  user_indexes i, user_ind_columns c, user_ind_expressions e, user_tab_cols tc
  	where i.table_name = c.table_name
        and   e.table_name = i.table_name
        and   e.index_name = i.index_name
    	and   i.index_name = c.index_name
        and   e.column_position = c.column_position
        and   c.table_name = tc.table_name
        and   c.column_name = tc.column_name
    	and   i.table_name like upper('&&Table_Name')
        and   i.partitioned = 'NO'
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) 
            no_merge(e) use_hash(e) */ 
        /* and now partitioned FBIs */ 
                i.index_name||' ('||ip.partition_name||') '||' (P:'||p.locality||' '||p.alignment||')' index_name,
                c.table_name,
                decode(tc.hidden_column,'NO',c.column_name,fkt_get_idx_expression(i.index_name,i.table_name,c.column_position))||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  user_indexes i, user_ind_partitions ip, user_tab_cols tc, 
              user_ind_columns c, user_part_indexes p, user_ind_expressions e
        where i.table_name = c.table_name
        and   e.table_name = i.table_name
        and   e.index_name = i.index_name
        and   e.column_position = c.column_position
        and   p.index_name = ip.index_name
        and   i.index_name = ip.index_name
        and   ip.index_name = c.index_name
        and   c.table_name = tc.table_name
        and   c.column_name = tc.column_name
        and   i.table_name like upper('&&Table_Name')
        and   ip.tablespace_name is not null /* Comp. part. Indexes */
 union
select  /*+ ordered 
            no_merge(i) 
            no_merge(ip) use_hash(ip)
            no_merge(c) use_hash(c) 
            no_merge(p) use_hash(p) 
            no_merge(e) use_hash(e) */ 
        /* and now sub-partitioned FBIs */ 
                i.index_name||' ('||ip.subpartition_name||') '||' (SP:'||p.locality||' '||p.alignment||')' index_name,
                c.table_name,
                decode(tc.hidden_column,'NO',c.column_name,fkt_get_idx_expression(i.index_name,i.table_name,c.column_position))||' '||decode(descend,'ASC','',descend) column_name,
                decode(i.table_type,'TABLE','TAB','CLUSTER','CLU','???') type,
                decode(i.index_type,'BITMAP','BIT',
                                    'CLUSTER','CLU',
                                    'IOT - TOP','IOT',
                                    'LOB','LOB',
                                     decode(i.uniqueness,'UNIQUE','YES',
                                                         'NONUNIQUE','NO')) UNI,
                ip.tablespace_name, i.distinct_keys, c.column_position
        from  user_indexes i, user_ind_subpartitions ip, user_tab_cols tc,
              user_ind_columns c, user_part_indexes p, user_ind_expressions e
        where i.table_name = c.table_name
        and   e.table_name = i.table_name
        and   e.index_name = i.index_name
        and   e.column_position = c.column_position
        and   p.index_name = ip.index_name
        and   i.index_name = ip.index_name
        and   ip.index_name = c.index_name
        and   c.table_name = tc.table_name
        and   c.column_name = tc.column_name
        and   i.table_name like upper('&&Table_Name')
order by 2, 5 desc, 1, 8;

ttitle off
clear break compute     
column index_name	CLEAR
column type		CLEAR
column uni              CLEAR
column tablespace_name	CLEAR
column table_name	CLEAR
column column_name	CLEAR
column distinct_keys	CLEAR
column column_position  CLEAR
undefine table_name
@temp.tmp
